==> iconify_icons-1.0.0-rc1.tar.gz: iconify_icons/src/Plugin/Field/FieldWidget/IconifyIconWidget.php <==
<?php

namespace Drupal\iconify_icons\Plugin\Field\FieldWidget;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\iconify_icons\IconifyServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'iconify_icon' widget.
 *
 * @FieldWidget(
 *   id = "iconify_icon_widget",
 *   module = "iconify_icons",
 *   label = @Translation("Iconify Icon"),
 *   field_types = {
 *     "iconify_icon"
 *   }
 * )
 */
class IconifyIconWidget extends WidgetBase {

  /**
   * Drupal configuration service container.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Drupal Iconify service.
   *
   * @var \Drupal\iconify_icons\IconifyServiceInterface
   */
  protected $iconify;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, ConfigFactory $config_factory, IconifyServiceInterface $iconify, AccountInterface $current_user) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->configFactory = $config_factory;
    $this->iconify = $iconify;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('config.factory'),
      $container->get('iconify_icons.iconify_service'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'collections' => [],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['search'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Icon Collections'),
      '#size' => 60,
      '#maxlength' => 128,
      '#placeholder' => $this->t('Filter collections'),
      '#attributes' => [
        'class' => ['iconify-icons-widget-checkboxes-filter'],
      ],
      '#attached' => [
        'library' => ['iconify_icons/default'],
      ],
    ];

    $element['collections'] = [
      '#type' => 'checkboxes',
      '#options' => $this->getOptions(),
      '#default_value' => $this->getSetting('collections'),
      '#description' => $this->t('Select the collections which are going to provide the set of icons. See @collectionIconsLink list.', [
        '@collectionIconsLink' => Link::fromTextAndUrl($this->t('the Iconify icon collections'), Url::fromUri('https://icon-sets.iconify.design/', [
          'attributes' => [
            'target' => '_blank',
          ],
        ]))->toString(),
      ]),
      '#attributes' => [
        'class' => [
          'iconify-icons-widget-collections',
        ],
      ],
      '#attached' => [
        'library' => [
          'iconify_icons/default',
        ],
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $selected_collections = $this->getSetting('collections');
    $summary[] = $this->t('Collections: @collections', [
      '@collections' => !empty($selected_collections) ? implode(', ', array_filter($selected_collections, static fn($value) => $value !== 0)) : 'All',
    ]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $cardinality = $this->fieldDefinition->getFieldStorageDefinition()->getCardinality();
    /** @var \Drupal\iconify_icons\Plugin\Field\FieldType\IconifyIcon $iconify_icon */
    $iconify_icon = $items[$delta];
    // Generate a unique key for the drupalSettings based on the delta.
    $settings_key = $items->getName() . '_' . $delta;
    // Get selected collections from widget settings.
    $selected_collections = $this->getSetting('collections');
    $collection = implode(',', array_filter($selected_collections, static fn($value) => $value !== 0));
    $settings = unserialize($iconify_icon->get('settings')->getValue() ?? '', ['allowed_classes']);

    $element['icon_name'] = [
      '#type' => 'iconify_icons',
      '#title' => $cardinality === 1 ? $this->fieldDefinition->getLabel() : $this->t('Icon Name'),
      '#default_value' => $iconify_icon->get('icon_name')->getValue(),
      '#required' => $element['#required'],
      '#size' => 50,
      '#attributes' => [
        'data-settings-key' => $settings_key,
      ],
      '#collections' => $collection,
      '#attached' => [
        'drupalSettings' => [
          'iconify_icons' => [
            $settings_key => [
              'icon_name' => $iconify_icon->get('icon_name')->getValue(),
              'icon_svg' => $iconify_icon->get('icon_svg')->getValue(),
            ],
          ],
        ],
      ],
    ];

    $element['settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Iconify icon settings'),
    ];

    $element['settings']['width'] = [
      '#type' => 'number',
      '#title' => $this->t('Width'),
      '#description' => $this->t('Icon dimensions in pixels. If only one dimension is specified, such as height, other dimension will be automatically set to match it.'),
      '#size' => 8,
      '#maxlength' => 8,
      '#min' => 1,
      '#max' => 99999,
      '#field_suffix' => t('pixels'),
      '#default_value' => $settings['width'] ?? 50,
    ];

    $element['settings']['height'] = [
      '#type' => 'number',
      '#title' => $this->t('Height'),
      '#description' => $this->t('Icon dimensions in pixels. If only one dimension is specified, such as height, other dimension will be automatically set to match it.'),
      '#size' => 8,
      '#maxlength' => 8,
      '#min' => 1,
      '#max' => 99999,
      '#field_suffix' => t('pixels'),
      '#default_value' => $settings['height'] ?? 50,
    ];

    $element['settings']['color'] = [
      '#type' => 'color',
      '#title' => $this->t('Color'),
      '#description' => $this->t('Icon color. Replaces current color with specific color, resulting in icon with hardcoded palette.'),
      '#default_value' => $settings['color'] ?? '',
    ];

    $element['settings']['flip'] = [
      '#type' => 'select',
      '#title' => $this->t('Flip'),
      '#empty_option' => $this->t('- None -'),
      '#options' => [
        'vertical' => $this->t('Vertical'),
        'horizontal' => $this->t('Horizontal'),
      ],
      '#description' => $this->t('Flip icon.'),
      '#default_value' => $settings['flip'] ?? '',
    ];

    $element['settings']['rotate'] = [
      '#type' => 'select',
      '#title' => $this->t('Rotate'),
      '#empty_option' => $this->t('- None -'),
      '#options' => [
        '90' => $this->t('90°'),
        '180' => $this->t('180°'),
        '270' => $this->t('270°'),
      ],
      '#description' => $this->t('Rotate icon by 90, 180 or 270 degrees.'),
      '#default_value' => $settings['rotate'] ?? '',
    ];

    $element['settings']['link_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL'),
      '#description' => $this->t('The URL to which the user should be redirected. This can be an internal URL like /node/1234 or an external URL like @url.', ['@url' => 'http://example.com']),
      '#default_value' => $settings['link_url'] ?? '',
      '#pattern' => '^(https?:\/\/[^\s]+|www\.[^\s]+|\/[^\s]+)$',
    ];

    $element['settings']['link_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link Text'),
      '#description' => $this->t('It will show a text close to the icon.'),
      '#default_value' => $settings['link_text'] ?? '',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $collection = $this->getSetting('collections');
    foreach ($values as $delta => &$item) {
      $item['delta'] = $delta;
      $item['selected_collection'] = $collection;
      // Take 'Icon name (collection name)', match the collection name from
      // inside the parentheses.
      // @see \Drupal\Core\Entity\Element\EntityAutocomplete::extractEntityIdFromAutocompleteInput
      if (preg_match('/(.+\\s)\\(([^\\)]+)\\)/', $item['icon_name'], $matches)) {
        $item['icon_sort_name'] = trim($matches[1]);
        $item['icon_collection'] = trim($matches[2]);
        $item['icon_svg'] = $this->iconify->generateSvg($item['icon_collection'], $item['icon_sort_name'], $item['settings']);
      }
      $item['settings'] = serialize(array_filter($item['settings']));
    }

    return $values;
  }

  /**
   * Generates a custom name for an icon collection.
   *
   * @param array $collection
   *   An associative array containing details of the icon collection.
   *   Expected keys are 'name', 'category', and 'total'.
   * @param string $collection_id
   *   The unique identifier for the icon collection.
   *
   * @return string
   *   A formatted string containing the collection name, category, total
   *   number of icons, and a link to view the icons.
   */
  protected function getCustomCollectionName(array $collection, string $collection_id): string {
    return sprintf(
      '<strong>%s</strong> - %s (%d) <a href="https://icon-sets.iconify.design/%s" target="_blank">See icons</a>',
      $collection['name'] ?? 'Unknown Name',
      $collection['category'] ?? 'Uncategorized',
      $collection['total'] ?? 0,
      $collection_id
    );
  }

  /**
   * Gets a sorted list of icon collections with custom names.
   *
   * @return array
   *   An associative array where the keys are collection IDs and the values
   *   are custom collection names.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   * @throws \JsonException
   */
  protected function getOptions(): array {
    // Fetch and sort collections by 'total' in descending order.
    $collections = $this->iconify->getCollections();
    uasort($collections, fn($a, $b) => $b['total'] <=> $a['total']);

    $options = [];

    foreach ($collections as $collection_id => $collection) {
      $options[$collection_id] = $this->getCustomCollectionName($collection, $collection_id);
    }

    return $options;
  }

}

==> iconify_icons-1.0.1.tar.gz: iconify_icons/src/Plugin/Field/FieldWidget/IconifyIconWidget.php <==
<?php

namespace Drupal\iconify_icons\Plugin\Field\FieldWidget;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\iconify_icons\IconifyServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'iconify_icon' widget.
 *
 * @FieldWidget(
 *   id = "iconify_icon_widget",
 *   module = "iconify_icons",
 *   label = @Translation("Iconify Icon"),
 *   field_types = {
 *     "iconify_icon"
 *   }
 * )
 */
class IconifyIconWidget extends WidgetBase {

  /**
   * Drupal configuration service container.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Drupal Iconify service.
   *
   * @var \Drupal\iconify_icons\IconifyServiceInterface
   */
  protected $iconify;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, ConfigFactory $config_factory, IconifyServiceInterface $iconify, AccountInterface $current_user) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->configFactory = $config_factory;
    $this->iconify = $iconify;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('config.factory'),
      $container->get('iconify_icons.iconify_service'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'collections' => [],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['search'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Icon Collections'),
      '#size' => 60,
      '#maxlength' => 128,
      '#placeholder' => $this->t('Filter collections'),
      '#attributes' => [
        'class' => ['iconify-icons-widget-checkboxes-filter'],
      ],
      '#attached' => [
        'library' => ['iconify_icons/default'],
      ],
    ];

    $element['collections'] = [
      '#type' => 'checkboxes',
      '#options' => $this->getOptions(),
      '#default_value' => $this->getSetting('collections'),
      '#description' => $this->t('Select the collections which are going to provide the set of icons. See @collectionIconsLink list.', [
        '@collectionIconsLink' => Link::fromTextAndUrl($this->t('the Iconify icon collections'), Url::fromUri('https://icon-sets.iconify.design/', [
          'attributes' => [
            'target' => '_blank',
          ],
        ]))->toString(),
      ]),
      '#attributes' => [
        'class' => [
          'iconify-icons-widget-collections',
        ],
      ],
      '#attached' => [
        'library' => [
          'iconify_icons/default',
        ],
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $selected_collections = array_filter($this->getSetting('collections'));
    $collections_summary = !empty($selected_collections) ? implode(', ', $selected_collections) : 'All';

    return [
      $this->t('Collections: @collections', ['@collections' => $collections_summary]),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $cardinality = $this->fieldDefinition->getFieldStorageDefinition()->getCardinality();
    /** @var \Drupal\iconify_icons\Plugin\Field\FieldType\IconifyIcon $iconify_icon */
    $iconify_icon = $items[$delta];
    // Generate a unique key for the drupalSettings based on the delta.
    $settings_key = $items->getName() . '_' . $delta;
    // Get selected collections from widget settings.
    $selected_collections = $this->getSetting('collections');
    $collection = implode(',', array_filter($selected_collections, static fn($value) => $value !== 0));
    $settings = unserialize($iconify_icon->get('settings')->getValue() ?? '', ['allowed_classes']);
    $icon = $iconify_icon->get('icon')->getValue() ?? '';
    // Extract the icon name and collection name using a regular expression.
    if ($iconDetails = $this->extractIconDetails($icon)) {
      [$icon_name, $icon_collection] = $iconDetails;
      $icon_svg = $this->iconify->generateSvg($icon_collection, $icon_name, $settings);
    }

    $element['icon'] = [
      '#type' => 'iconify_icons',
      '#title' => $cardinality === 1 ? $this->fieldDefinition->getLabel() : $this->t('Icon Name'),
      '#default_value' => $iconify_icon->get('icon')->getValue(),
      '#required' => $element['#required'],
      '#size' => 50,
      '#attributes' => [
        'data-settings-key' => $settings_key,
      ],
      '#collections' => $collection,
      '#attached' => [
        'drupalSettings' => [
          'iconify_icons' => [
            $settings_key => [
              'icon_name' => $icon,
              'icon_svg' => $icon_svg ?? '',
            ],
          ],
        ],
      ],
    ];

    $element['settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Iconify icon settings'),
    ];

    $element['settings']['width'] = [
      '#type' => 'number',
      '#title' => $this->t('Width'),
      '#description' => $this->t('Icon dimensions in pixels. If only one dimension is specified, such as height, other dimension will be automatically set to match it.'),
      '#size' => 8,
      '#maxlength' => 8,
      '#min' => 1,
      '#max' => 99999,
      '#field_suffix' => t('pixels'),
      '#default_value' => $settings['width'] ?? 50,
    ];

    $element['settings']['height'] = [
      '#type' => 'number',
      '#title' => $this->t('Height'),
      '#description' => $this->t('Icon dimensions in pixels. If only one dimension is specified, such as height, other dimension will be automatically set to match it.'),
      '#size' => 8,
      '#maxlength' => 8,
      '#min' => 1,
      '#max' => 99999,
      '#field_suffix' => t('pixels'),
      '#default_value' => $settings['height'] ?? 50,
    ];

    $element['settings']['color'] = [
      '#type' => 'color',
      '#title' => $this->t('Color'),
      '#description' => $this->t('Icon color. Replaces current color with specific color, resulting in icon with hardcoded palette.'),
      '#default_value' => $settings['color'] ?? '',
    ];

    $element['settings']['flip'] = [
      '#type' => 'select',
      '#title' => $this->t('Flip'),
      '#empty_option' => $this->t('- None -'),
      '#options' => [
        'vertical' => $this->t('Vertical'),
        'horizontal' => $this->t('Horizontal'),
      ],
      '#description' => $this->t('Flip icon.'),
      '#default_value' => $settings['flip'] ?? '',
    ];

    $element['settings']['rotate'] = [
      '#type' => 'select',
      '#title' => $this->t('Rotate'),
      '#empty_option' => $this->t('- None -'),
      '#options' => [
        '90' => $this->t('90°'),
        '180' => $this->t('180°'),
        '270' => $this->t('270°'),
      ],
      '#description' => $this->t('Rotate icon by 90, 180 or 270 degrees.'),
      '#default_value' => $settings['rotate'] ?? '',
    ];

    $element['settings']['link_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL'),
      '#description' => $this->t('The URL to which the user should be redirected. This can be an internal URL like /node/1234, an external URL like @url, or an anchor like #main-content.', ['@url' => 'http://example.com']),
      '#default_value' => $settings['link_url'] ?? '',
      '#pattern' => '^(https?:\/\/[^\s]+|www\.[^\s]+|\/[^\s]+|#[a-zA-Z0-9_-]+)$',
    ];

    $element['settings']['link_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link Text'),
      '#description' => $this->t('It will show a text close to the icon.'),
      '#default_value' => $settings['link_text'] ?? '',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $collection = $this->getSetting('collections');
    foreach ($values as $delta => &$item) {
      $item['delta'] = $delta;
      $item['selected_collection'] = $collection;
      $item['settings'] = serialize(array_filter($item['settings']));
    }

    return $values;
  }

  /**
   * Generates a custom name for an icon collection.
   *
   * @param array $collection
   *   An associative array containing details of the icon collection.
   *   Expected keys are 'name', 'category', and 'total'.
   * @param string $collection_id
   *   The unique identifier for the icon collection.
   *
   * @return string
   *   A formatted string containing the collection name, category, total
   *   number of icons, and a link to view the icons.
   */
  protected function getCustomCollectionName(array $collection, string $collection_id): string {
    return sprintf(
      '<strong>%s</strong> - %s (%d) <a href="https://icon-sets.iconify.design/%s" target="_blank">See icons</a>',
      $collection['name'] ?? 'Unknown Name',
      $collection['category'] ?? 'Uncategorized',
      $collection['total'] ?? 0,
      $collection_id
    );
  }

  /**
   * Gets a sorted list of icon collections with custom names.
   *
   * @return array
   *   An associative array where the keys are collection IDs and the values
   *   are custom collection names.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   * @throws \JsonException
   */
  protected function getOptions(): array {
    // Fetch and sort collections by 'total' in descending order.
    $collections = $this->iconify->getCollections();
    uasort($collections, fn($a, $b) => $b['total'] <=> $a['total']);

    $options = [];

    foreach ($collections as $collection_id => $collection) {
      $options[$collection_id] = $this->getCustomCollectionName($collection, $collection_id);
    }

    return $options;
  }

  /**
   * Extracts icon name and collection name from the given string.
   *
   * @param string $icon
   *   The icon string in the format 'Icon name (collection name)'.
   *
   * @return array|null
   *   An array containing the icon name and collection name, or null if not
   *   matched.
   */
  protected function extractIconDetails(string $icon): ?array {
    if (preg_match('/(.+)\s\(([^)]+)\)/', $icon, $matches)) {
      return [trim($matches[1]), trim($matches[2])];
    }

    return NULL;
  }

}
